/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions.utils;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.web.actions.AddCollectionAction;
import org.wso2.registry.servlet.ConsoleURLMapper;

import javax.servlet.http.HttpServletRequest;

public class ActionInvoker {

    public static final String SUCCESS = "SUCCESS";
    public static final String ERROR = "ERROR";

    public static String invokeActions(HttpServletRequest request) {

        String contextRoot = getContextRoot(request);

        String uri = request.getRequestURI();
        String query = request.getQueryString();

        String adminPart = uri.substring(contextRoot.length());
        if (!adminPart.startsWith(RegistryConstants.PATH_SEPARATOR + ConsoleURLMapper.WEB_PATH)) {
            return ERROR;
        }

        String path;
        if (uri.endsWith("") || uri.endsWith(ConsoleURLMapper.WEB_PATH + RegistryConstants.PATH_SEPARATOR)) {
            path = RegistryConstants.ROOT_PATH;

        } else {
            path = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + ConsoleURLMapper.WEB_PATH).
                    length(), uri.length());
        }

        if (query != null) {
            if (query.equals("addCollection")) {

                String collectionName = (String) request.getSession().getAttribute("collectionName");
                String mediaType = (String) request.getSession().getAttribute("mediaType");
                AddCollectionAction addCollectionAction = new AddCollectionAction();
                addCollectionAction.setParentPath(path);
                addCollectionAction.setCollectionName(collectionName);
                addCollectionAction.setMediaType(mediaType);

                try {
                    return addCollectionAction.execute();
                } catch (RegistryException e) {
                    e.printStackTrace();
                    return ERROR;
                }
            }
        }

        return SUCCESS;
    }

    public static String getContextRoot(HttpServletRequest req) {

        String contextPath = req.getContextPath();
        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        
        return contextPath;
    }
}
